// / <reference path="./types/jsx.d.ts" />
import { Pico } from "./deps.ts";

import config from "./config.ts";

const React = {
   createElement: Pico.h.bind(Pico),
};

class StringReader implements Deno.Reader {
   private data: Uint8Array;
   private offset = 0;
   constructor(text: string) {
      this.data = new TextEncoder().encode(text);
   }

   async read(p: Uint8Array): Promise<number | null> {
      if (this.offset >= this.data.byteLength) return null;

      const forLength = Math.min(p.length, this.data.length - this.offset);

      for (let i = 0; i < forLength; i++) {
         p[i] = this.data[i + this.offset];
      }

      this.offset += forLength;

      return forLength;
   }
}

type ELM = any;

type Component = () => ELM;

const componentCache = new Map<string, Component>();

async function loadComponent(name: string) {
   let mod = componentCache.get(name);
   if (!mod || config.general.dev) {
      mod = (await import(`./views/${name}.tsx`)).default;
      if (!mod) throw new Error("Invalid component " + name);
      componentCache.set(name, mod);
   }

   return mod;
}

const preLoad = ["index", "package", "browse_folder", "browse_file"];

preLoad.forEach((page) =>
   loadComponent(page).catch((err) => console.error("Error preloading", page))
);

export default async function render(
   name: string,
   data: any
): Promise<Deno.Reader> {
   const Component = await loadComponent(name);

   //@ts-ignore
   const res = await Pico.renderSSR(<Component {...data} />);

   return new StringReader("<!DOCTYPE html>\n" + res);
}
